function [cel, result] = FilterOptimize(H, HT, LorH, inivar)

%
%  FilterOptimize(H, HT, LorH, option, inivar)
%
%    The optimization on the vector filters, through the gradient
%    decending search algorithm.
%    
%    H: refinement mask of H or G, H(z)=sum{P_k*z^k}/2;
%    HT: refinement mask of H~ or G~, H(z)=sum{Q_k*z^k}/2;
%    LorH: flag indicates if the filter is lowpass or highpass
%          0 - lowpass filter
%          1 - highpass filter
%    inivar: the initial value in search algorithm
%

syms w real;

HW = subs(H,'z',exp(-i*w))
HTW = subs(HT,'z',exp(-i*w))

cel = FindUnknowVar(HW, HTW)
if isempty(cel)
     return;
end

initial = inivar*ones(length(cel),1);
min = initial;
grad = initial;
step = 0.001;
count = 0;
dfactor = 1;
dcount = 0;
accuracy = 0.001;
stopcount = 40;
lastgrad = grad;

while (1)
   initial = min;

   for k = 1:length(cel)

     % NUMERICAL INTEGRAL
     grad(k) = 0;
     point = initial;
     point(k) = point(k) - 2*step;
     f1 = NumerIntegral(HW,HTW,cel,point,LorH);
     point = initial;
     point(k) = point(k) - 1*step;
     f2 = NumerIntegral(HW,HTW,cel,point,LorH);
     point = initial;
     point(k) = point(k) + 1*step;
     f3 = NumerIntegral(HW,HTW,cel,point,LorH);
     point = initial;
     point(k) = point(k) + 2*step;
     f4 = NumerIntegral(HW,HTW,cel,point,LorH);

     grad(k) = (-f4+8*f3-8*f2+f1)/(12*step);
   end

   grad
   if (norm(grad) < accuracy)  
     initial = initial + grad;
     break;
   elseif (norm(grad) > 30)
     grad = grad/norm(grad)/5/dfactor;
   elseif (norm(grad) > 10)
     grad = grad/norm(grad)/10/dfactor;
   elseif (norm(grad) > 1)
     grad = grad/norm(grad)/20/dfactor;
   elseif (norm(grad) > 0.5)
     grad = grad/40/dfactor;
   elseif (norm(grad) > 0.1)
     grad = grad/50/dfactor;
   elseif (norm(grad) > 0.01)
     grad = grad/60/dfactor;
   elseif (norm(grad)>0.001)
     grad = grad/100/dfactor;
   else
     grad = grad/10;
   end 

   min = initial - grad
   for k = 1:length(grad)
     if (grad(k)*lastgrad(k) > 0)
        break;
     end
     if (k == length(grad))
       dcount = dcount + 1;
     end
   end
   
   if (dcount > 2)
     dfactor = dfactor*2;
     dcount = 0;
   end
     lastgrad = grad;

   count = count + 1;

   if count > stopcount
      min = (initial + min)/2;
      break;
   end
end

result = min;
